# frozen_string_literal: true

module SpiritHub
  class Setting
    DEFAULT = {
      object_name: 'Projet',
      organism_type: 'Agence'
    }.freeze

    # Define accessor methods for each setting
    #
    # Example for object_name:
    #   Class method: Setting.main_object_name
    #   Instance method: setting.object_name
    #   Instance method: setting.object_name=
    #
    # @param key [Symbol] The key of the setting
    # @param value [String] The default value of the setting
    # @return [String] The current value of the setting
    DEFAULT.each do |key, value|
      define_method(key) { @settings.values[key] || value }
      define_method("#{key}=", ->(val) { @settings.values[key] = val })
      define_singleton_method("main_#{key}") { cached_value(key) }
    end

    delegate :id, :save, :update, to: :@settings

    def self.where(*args)
      ::Setting.where(*args)
    end

    def self.cache_key(key)
      "spirit_hub:setting:#{key}"
    end

    def self.cached_value(key)
      Rails.cache.fetch(cache_key(key)) do
        instance = new
        instance.respond_to?(key) ? instance.public_send(key) : instance[key]
      end
    end

    def self.clear_cache!
      Rails.cache.delete_matched(cache_key('*'))
    end

    def initialize
      @settings = ::Setting.get_settings_for(:spirit_hub)
    end

    def [](key)
      @settings.values[key]
    end

    def []=(key, value)
      @settings.values[key] = value
    end

    def model_name
      self.class
    end

    def self.param_key
      :spirit_hub_setting
    end
  end
end
